@@ -15,7 +15,7 @@ ssl._create_default_https_context = ssl._create_unverified_context
 from termcolor import cprint
 from inspect import currentframe
 
-__version__ = '0.1.48'
+__version__ = '0.1.51'
 
 __info__ = {
     '2022/01/04: 0.1.18 加入 __info__。',
@@ -42,10 +42,13 @@ __info__ = {
     '2022/08/12: 0.1.42 fix redis 跳出問題',
     '2022/08/18: 0.1.43 修改 redis_msg_sender(), 增加 channel參數',
     '2022/08/19: 0.1.44 修改Tredis，改成兩個thread，避免任何block',
-    '2022/08/30: 0.1.48 修改 get_all_stock_code_name_dict() 加入account'
+    '2022/08/30: 0.1.48 修改 get_all_stock_code_name_dict() 加入account',
+    '2022/09/05: 0.1.49 tplaysound 如果沒有找到檔案不播放，不然會crash',
+    '2022/09/06: 0.1.50 加入 crypto 模組',
+    '2022/10/23: 0.1.51 get_all_stock_code_name_dict() 加入 _api 參數',
+    '2022/10/25: 0.1.52 price_ticks_offset_AB_dec() 還有其他改成Decimal',
 }
 
-
 request_headers = {
     'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.95 Safari/537.36'
 }
@@ -112,7 +115,7 @@ def __LINE__():
     return cf.f_back.f_lineno
 
 
-def get_all_stock_code_name_dict(id='PAPIUSER08', password='2222'):
+def get_all_stock_code_name_dict(id='PAPIUSER08', password='2222', _api=None):
     today = datetime.datetime.today()
 
     cache_file = f'all_stock_code-{today.strftime("%Y%m%d")}.json'
@@ -121,13 +124,16 @@ def get_all_stock_code_name_dict(id='PAPIUSER08', password='2222'):
 
         import shioaji as sj
 
-        def simu_login(id=id, password=password):
-            print(f'使用測試帳號 {id} {password}')
-            api = sj.Shioaji()
-            api.login(id, password, contracts_cb=lambda security_type: print(f"{repr(security_type)} fetch done."))
-            return api
+        if _api is None:
+            def simu_login(id=id, password=password):
+                print(f'使用測試帳號 {id} {password}')
+                api = sj.Shioaji()
+                api.login(id, password, contracts_cb=lambda security_type: print(f"{repr(security_type)} fetch done."))
+                return api
 
-        api = simu_login()
+            api = simu_login()
+        else:
+            api = _api
 
         all_code_name_dir = {}
         for x in api.Contracts.Stocks.OTC:
@@ -145,7 +151,6 @@ def get_all_stock_code_name_dict(id='PAPIUSER08', password='2222'):
         with open(cache_file, 'r') as fp:
             all_code_name_dir = json.load(fp)
 
-
     return all_code_name_dir
 
 
@@ -409,7 +414,10 @@ class timeIt:
 ########################
 from decimal import Decimal
 
+
 def get_current_price_tick_dec(price: Decimal, down=False):
+    if isinstance(price, int) or isinstance(price, float):
+        price = Decimal(str(price))
     # print(f'get_current_price_tick: {price}')
     if down:
         if price <= Decimal('10'):
@@ -441,6 +449,12 @@ def get_current_price_tick_dec(price: Decimal, down=False):
 
 # price1 should be .LE. to price2
 def get_ticks_between_dec(_price1: Decimal, _price2: Decimal):
+    if isinstance(_price1, int) or isinstance(_price1, float):
+        price = Decimal(str(_price1))
+
+    if isinstance(_price2, int) or isinstance(_price2, float):
+        price = Decimal(str(_price2))
+
     if _price2 < _price1:
         price1 = _price2
         price2 = _price1
@@ -458,21 +472,33 @@ def get_ticks_between_dec(_price1: Decimal, _price2: Decimal):
     return ticks
 
 
-# TODO: Use decimal
-def price_ticks_offset_dec(price, ticks):
-    current_tick = get_current_price_tick(price)
-    price = round(price - (price + 0.001) % current_tick, 2)
-    # print(f'normalized price: {price}')
-    if ticks == 0:
-        return price
-    step = 1 if ticks > 0 else -1
+
+def price_ticks_offset_AB_dec(price: Decimal, ticks):
+    if isinstance(price, int) or isinstance(price, float):
+        price = Decimal(str(price))
+    list_ask = []
+    list_bid = []
+    price_ask = price
+    price_bid = price
+    for i in range(ticks):
+        price_ask += get_current_price_tick_dec(price_ask, down=False)
+        price_bid -= get_current_price_tick_dec(price_bid, down=True)
+        list_ask.append(price_ask)
+        list_bid.append(price_bid)
+    return list_ask, list_bid
+
+
+
+def price_ticks_offset_dec(price: Decimal, ticks):
+    if isinstance(price, int) or isinstance(price, float):
+        price = Decimal(str(price))
+    step = -1 if ticks < 0 else 1
     for i in range(0, ticks, step):
-        current_tick = get_current_price_tick(price, down=True if step == -1 else False)
-        # print(i, price, current_tick)
-        price += current_tick * step
-    return round(price, 3)
+        price += step * get_current_price_tick_dec(price, down=True if step < 0 else False)
+    return price
 
 
+# TODO: 這個還有問題，並不是Decimal ，目前沒時間修 2022/10/23
 def get_limit_up_and_down_price_dec(price):
     limit_up = price * 1.1
     tick = get_current_price_tick(limit_up)
@@ -765,7 +791,6 @@ def build_dealer_downloader(date=datetime.datetime(year=2020, month=1, day=1), t
             os.unlink(dealer_csv_path + '_')
 
 
-
 def get_trade_days(date_start: Union[str, datetime.datetime] = '2018-01-01',
                    date_end: Union[str, datetime.datetime] = '2022-07-31') -> list:
     date_start = date_start if isinstance(date_start, str) else date_start.strftime('%Y-%m-%d')
@@ -797,9 +822,32 @@ def get_trade_days(date_start: Union[str, datetime.datetime] = '2018-01-01',
 if __name__ == '__main__':
     from time import sleep
 
-    print(get_all_stock_code_name_dict('H121933940', '123edcxzaqws'))
-
     if True:
+        print(100.5, 1, price_ticks_offset_dec(100.5, 1))
+        print(100.5, -1, price_ticks_offset_dec(100.5, -1))
+        print(100, 1, price_ticks_offset_dec(100, 1))
+        print(100, -1, price_ticks_offset_dec(100, -1))
+        print(99.9, 1, price_ticks_offset_dec(99.9, 1))
+        print(99.9, -1, price_ticks_offset_dec(99.9, -1))
+
+        print(50.1, 1, price_ticks_offset_dec(50.1, 1))
+        print(50.1, -1, price_ticks_offset_dec(50.1, -1))
+        print(50, 1, price_ticks_offset_dec(50, 1))
+        print(50, -1, price_ticks_offset_dec(50, -1))
+        print(49.5, 1, price_ticks_offset_dec(49.5, 1))
+        print(49.5, -1, price_ticks_offset_dec(49.5, -1))
+
+        print(100, 6, price_ticks_offset_AB_dec(100, 6))
+        print(99.8, 6, price_ticks_offset_AB_dec(99.8, 6))
+        print(101.5, 6, price_ticks_offset_AB_dec(101.5, 6))
+        print(50.2, 6, price_ticks_offset_AB_dec(50.2, 6))
+        print(50, 6, price_ticks_offset_AB_dec(50, 6))
+        print(49.9, 6, price_ticks_offset_AB_dec(49.9, 6))
+
+    if False:
+        print(get_all_stock_code_name_dict('H121933940', '123edcxzaqws'))
+
+    if False:
         ret = get_trade_days()
         print(ret)
 
